@@ -600,7 +600,7 @@ function (a::QQMPolyRingElem)(vals::NCRingElement...)
 end
 
 function evaluate(a::QQMPolyRingElem, bs::Vector{QQMPolyRingElem})
-  allequal(map(parent, bs)) || error("parents do not match")
+  @req allequal(map(parent, bs)) "parents do not match"
   R = parent(a)
   S = parent(bs[1])
 
@@ -614,7 +614,7 @@ function evaluate(a::QQMPolyRingElem, bs::Vector{QQMPolyRingElem})
 end
 
 function evaluate(a::QQMPolyRingElem, bs::Vector{QQPolyRingElem})
-  allequal(map(parent, bs)) || error("parents do not match")
+  @req allequal(map(parent, bs)) "parents do not match"
   R = parent(a)
   S = parent(bs[1])
 
